/** \file RTDomainMEM.cpp 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "RTDomainMEM.h"

namespace rtaixml_domain
{
	RTDomainMEM::RTDomainMEM() 
	{ 
		if(IS_DEBUG) std::cout<<"|| RTDomainMEM::RTDomainMEM() ||"<<std::endl;
		Initialize();
		if(IS_DEBUG) std::cout<<"|| End of RTDomainMEM::RTDomainMEM() ||"<<std::endl;
	}

	RTDomainMEM::~RTDomainMEM()
	{	
	}
	
	unsigned int RTDomainMEM::NumberOfActiveProcesses()
	{
		return vAllActors.size();
	}

	//Search by SessionId, for the othe searcg use the following methods. This method looks for the session id and
	// if it does not found it returns an empty ones.
	ActorDomainRecord RTDomainMEM::GetActor(std::string sSessionId)
	{
		ActorDomainRecord adrEmpty;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(vAllActors.at(i).getSessionId()==sSessionId)
				return vAllActors.at(i);;
		return adrEmpty;
	}

	//short: 0 - SlaveName, 1 - SessionId, 2 - TargetName, 3 - (non ammesso)PortXML
	ActorDomainRecord RTDomainMEM::GetActor(std::string sValue, short Choice)
	{
		ActorDomainRecord adrEmpty;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			switch(Choice)
			{
				case 0:
					if(vAllActors.at(i).getSlaveName()==sValue)
						return vAllActors.at(i);;
					break;
				case 1:
					if(vAllActors.at(i).getSessionId()==sValue)
						return vAllActors.at(i);;
					break;
				case 2:
					if(vAllActors.at(i).getTargetName()==sValue)
						return vAllActors.at(i);;
					break;
				default:
					return adrEmpty;
			}
		return adrEmpty;
	}

	//XML port and Socket port are alternated.
	int RTDomainMEM::GetFreePort()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::GetFreePort ||" << std::endl;
		int iFreePort = iBasePortXML+1;
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - starting port: " << iFreePort << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - check for available port - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int i = 0; (unsigned int)i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - used port ("<< i <<"): " << vAllActors.at(i).getPortXML() << std::endl;
			if(vAllActors.at(i).getPortXML()==iFreePort)
			{
				iFreePort+=2;
				if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - new port:" << iFreePort  << std::endl;
				i=-1;
			}
		}
		if(iFreePort>iMaxPortXML) {
			if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - No more free ports."  << std::endl;
			iFreePort = 0;
		}
	
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreePort - choosen port: " << iFreePort  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::GetFreePort ||" << std::endl;
		return iFreePort;
	}
	
	
	std::string RTDomainMEM::GetFreeSlaveName()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::GetFreeSlaveName ||" << std::endl;
		std::string sRandom=get_random_string(6);
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSlaveName - random slave name: " << sRandom << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSlaveName - check for available slave name - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int j = 0; j<(int)vAllActors.size();j++) {
			if((vAllActors.at(j).getSlaveName()).compare(sRandom)==0 || IsThereATarget(sRandom))
			{
				sRandom=get_random_string(6);
				if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSlaveName - new random slave name:" << sRandom  << std::endl;
				j=-1;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSlaveName - choosen slave name: " << sRandom  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::GetFreeSlaveName ||" << std::endl;
		return sRandom;
	}
	
	std::string RTDomainMEM::GetFreeSessionId()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::GetFreeSessionId ||" << std::endl;
		std::string sRandom=get_random_string(12);
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSessionId - random session: " << sRandom << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSessionId - check for available session id - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int j = 0; j<(int)vAllActors.size();j++) {
			if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSessionId - j = "<< j <<  std::endl;
			if((vAllActors.at(j).getSessionId()).compare(sRandom)==0)
			{
				sRandom=get_random_string(DEFAULT_SESSION_LENGHT);
				if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSessionId - new random session:" << sRandom  << std::endl;
				j=-1;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainMEM::GetFreeSessionId - choosen session id: " << sRandom  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::GetFreeSessionId ||" << std::endl;
		return sRandom;
	}
		
	bool RTDomainMEM::AddActor(ActorDomainRecord adrNew)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::AddActor ||" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - Checking the actor existence - vAllActors.size()="<< vAllActors.size() << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - Actor n."<< i << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - SlaveName: "<< vAllActors.at(i).getSlaveName() << "<->" << adrNew.getSlaveName() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - SessionID: "<< vAllActors.at(i).getSessionId() << "<->" << adrNew.getSessionId() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - TargetName: "<< vAllActors.at(i).getTargetName() << "<->" << adrNew.getTargetName() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - XML Port: "<< vAllActors.at(i).getPortXML() << "<->" << adrNew.getPortXML() << std::endl;

			if(IS_DEBUG) 
			{
				if(adrNew.getSlaveName()==vAllActors.at(i).getSlaveName())
					std::cout<<" ++++++++++++++++++++++ Error SlaveName"<<std::endl;			
				if(adrNew.getSessionId()==vAllActors.at(i).getSessionId())
					std::cout<<" ++++++++++++++++++++++ Error SessionID"<<std::endl;			
				if(adrNew.getTargetName()==vAllActors.at(i).getTargetName())
					std::cout<<" ++++++++++++++++++++++ Error TargetName"<<std::endl;			
				if(adrNew.getPortXML()==vAllActors.at(i).getPortXML())
					std::cout<<" ++++++++++++++++++++++ Error PortName"<<std::endl;			
			}
						
			if(adrNew.getSlaveName()==vAllActors.at(i).getSlaveName() ||
				adrNew.getSessionId()==vAllActors.at(i).getSessionId() ||
				adrNew.getTargetName()==vAllActors.at(i).getTargetName() ||
				adrNew.getPortXML()==vAllActors.at(i).getPortXML())
			{
				if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - vAllActors.size()="<< vAllActors.size() << std::endl;
				if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - The actor is already registered in the rtaixml domain." << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::AddActor ||" << std::endl;
				return false;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - Inserting new actor in the rtaixml domain..." << std::endl;
		vAllActors.push_back(adrNew);
		if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - ...done" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::AddActor - vAllActors.size()="<< vAllActors.size() << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::AddActor ||" << std::endl;
		return true;
	}

	bool RTDomainMEM::AddActorAndChoosePort(ActorDomainRecord adrNew)
	{
		adrNew.setPortXML(GetFreePort());
		return AddActor(adrNew);
	}

	//in order:  sSlaveName sSessionId sTargetName sScopeName sDigitalName
	bool RTDomainMEM::AddNewActorAndChoosePort(	std::string sTargetName_new)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::AddNewActorAndChoosePort ||" << std::endl;
		if(CheckTargetSlaveManager(sTargetName_new))
		{
			if(IS_DEBUG) std::cout << "RTDomainMEM::AddNewActorAndChoosePort - Target: "<< sTargetName_new << " already managed by "<< getSlaveName(sTargetName_new,0) << "slave server." << std::endl;
			//If multiple slave for one target not allowed!
			if(!cpSecure->MultipleSlaveOneTarget()) {
				if(IS_DEBUG) std::cout << "RTDomainMEM::AddNewActorAndChoosePort - Multiple target management NOT allowed." << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::AddNewActorAndChoosePort ||" << std::endl;
				return false;
			}
			else
			{
				if(IS_DEBUG) std::cout << "RTDomainMEM::AddNewActorAndChoosePort - Multiple target management allowed!" << std::endl;
			}
		}
		ActorDomainRecord adrNew(GetFreeSlaveName(), GetFreeSessionId(), sTargetName_new, /*sScopeName_new, sDigitalName_new,*/ GetFreePort(),"");
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::AddNewActorAndChoosePort ||" << std::endl;
		return AddActor(adrNew);
	}
	

	bool RTDomainMEM::RemoveActor(ActorDomainRecord adrRemove)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::RemoveActor ||" << std::endl;
		if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " and Slave server: " << adrRemove.getSlaveName() << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(adrRemove.getSlaveName()==vAllActors.at(i).getSlaveName() &&
				adrRemove.getSessionId()==vAllActors.at(i).getSessionId() &&
				adrRemove.getTargetName()==vAllActors.at(i).getTargetName() &&
				adrRemove.getPortXML()==vAllActors.at(i).getPortXML())
			{
				std::vector<ActorDomainRecord>::iterator the_iterator;
				the_iterator = vAllActors.begin();
				vAllActors.erase(the_iterator+i);
				if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " found!" << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::RemoveActor ||" << std::endl;
				return true;
			}
		if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " NOT present!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::RemoveActor ||" << std::endl;
		return false;
	}

	bool RTDomainMEM::RemoveActor(std::string sTaskNameToRemove)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::RemoveActor ||" << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(sTaskNameToRemove==vAllActors.at(i).getTargetName())
			{
				std::vector<ActorDomainRecord>::iterator the_iterator;
				the_iterator = vAllActors.begin();
				vAllActors.erase(the_iterator+i);
				if(IS_DEBUG) std::cout << "Target: " << sTaskNameToRemove << " found!" << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::RemoveActor ||" << std::endl;
				return true;
			}
		if(IS_DEBUG) std::cout << "Target: " << sTaskNameToRemove << " NOT present!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::RemoveActor ||" << std::endl;
		return false;
	}

	bool RTDomainMEM::RemoveAllActor(ActorDomainRecord)
	{
		vAllActors.empty();
		return true;
	}
	
	bool RTDomainMEM::StopRemoveSlaveServer(std::string sSlaveNameToStop) {
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::StopSlaveServer ||" << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(sSlaveNameToStop==vAllActors.at(i).getSlaveName())
			{
				std::vector<ActorDomainRecord>::iterator the_iterator;
				the_iterator = vAllActors.begin();
				
				XmlRpc::XmlRpcClient c("localhost", vAllActors.at(i).getPortXML());
				XmlRpc::XmlRpcValue noArgs, result;
				noArgs[0]=vAllActors.at(i).getSessionId();
				noArgs[1]="none";
				try {
					if (c.execute("Stop_Server", noArgs, result))
						if(IS_DEBUG) std::cout << result << std::endl; 		
					else {
						if(IS_DEBUG) std::cout << "Error!" << std::endl; 		
					}
				}catch (XmlRpc::XmlRpcException ex) {
					if(IS_DEBUG) std::cout << "Error in connection with the slave server.." << std::endl; 		
				}
				
				vAllActors.erase(the_iterator+i);
				if(IS_DEBUG) std::cout << "Slave server: " << sSlaveNameToStop << " found!" << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::RemoveActor ||" << std::endl;
				return true;
			}
		if(IS_DEBUG) std::cout << "Slave server: " << sSlaveNameToStop << " NOT present!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::StopSlaveServer ||" << std::endl;
		return false;	}

	
	std::string RTDomainMEM::getSessionId(std::string sTaskName)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::getSessionId ||" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::getSessionId - check for session id - vAllActors.size() = " << vAllActors.size() << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainMEM::getSessionId - Target name to find: " << sTaskName << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainMEM::getSessionId - session id ("<< i <<"): " << vAllActors.at(i).getSessionId() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainMEM::getSessionId - target name ("<< i <<"): " << vAllActors.at(i).getTargetName() << std::endl;
			if(sTaskName==vAllActors.at(i).getTargetName())
			{
				if(IS_DEBUG) std::cout << "Found - Session ID: " << vAllActors.at(i).getSessionId() << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::getSessionId ||" << std::endl;
				return vAllActors.at(i).getSessionId();
			}
		}
		if(IS_DEBUG) std::cout << "NOT Found!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainMEM::getSessionId ||" << std::endl;
		return "";

	}
	
	//0 TaskName, 1 SessionId
	std::string RTDomainMEM::getSlaveName(std::string sName,short sChoice)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if( (sName==vAllActors.at(i).getTargetName() && sChoice == 0) ||
				(sName==vAllActors.at(i).getSessionId() && sChoice == 1) )
			{
				return vAllActors.at(i).getSlaveName();
			}
		return "";
	}

	int RTDomainMEM::getPortXMLSlave(std::string sName, short sChoice)
	{
		if(IS_DEBUG) { std::cout<<"RTDomainMEM::getPortXMLSlave(): Name: "<< sName << " - Choice: "<< sChoice << std::endl;std::fflush(stdout); }
		if(IS_DEBUG) { std::cout<<"RTDomainMEM::getPortXMLSlave(): vAllActors.size(): "<< vAllActors.size() << std::endl;std::fflush(stdout); }
		for(unsigned int i = 0; i<vAllActors.size();i++)
		{
			if(IS_DEBUG) { std::cout<<"RTDomainMEM::getPortXMLSlave(): i: "<< i << std::endl;std::fflush(stdout); }
			if( (sName==vAllActors.at(i).getTargetName() && sChoice == 0) ||
				(sName==vAllActors.at(i).getSessionId() && sChoice == 1) )
			{
				return vAllActors.at(i).getPortXML();
			}
		}
		if(IS_DEBUG) { std::cout<<"RTDomainMEM::getPortXMLSlave(): No Actors!"<< std::endl;std::fflush(stdout); }
		return 0;
	}
	
	std::vector< SlaveInfo > RTDomainMEM::getAllSlaveInfo()
	{
		std::vector< SlaveInfo > vSlaveInfos;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			SlaveInfo appo( vAllActors.at(i).getSessionId(), vAllActors.at(i).getPortXML() ); 
			vSlaveInfos.push_back( appo );
		}
		return vSlaveInfos;
	}
	
		
	std::string RTDomainMEM::getTaskName(std::string sSessionId)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(sSessionId==vAllActors.at(i).getSessionId())
			{
				return vAllActors.at(i).getTargetName();
			}
		return "";
	}

	void RTDomainMEM::PrintCurrentStatus()
	{
		std::cout << "================================================================================" << std::endl;
		std::cout << "\t\t\t INFO ABOUT RTAIXML DOMAIN :\t\t" << std::endl;
		std::cout << "================================================================================" << std::endl;
		std::cout << "Master Server Name :\t\t" << sMasterName << std::endl;
		std::cout << "XML master communication port :\t" << iBasePortXML << std::endl;
		std::cout << "XML slave starting port :\t" << iBasePortXML+1 << std::endl;
		std::cout << "XML slave port interval:\t" << iMaxPortXML-iBasePortXML << std::endl;
		std::cout << "Security policy :\t\t" << cpSecure->StringPrintPolicy() << std::endl;
		std::cout << std::endl;
		std::cout << "Number of running processes :\t" << NumberOfActiveProcesses() << std::endl;
		if((short)NumberOfActiveProcesses()>0)
		{
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		std::cout << "Slave Server Name\tSession ID\t\tTarget Name\t\tXML Port" << std::endl;
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		for(unsigned short i=0;i<NumberOfActiveProcesses();i++)
		{
		std::cout << vAllActors.at(i).getSlaveName() << "\t\t\t" << vAllActors.at(i).getSessionId() << "\t\t" << vAllActors.at(i).getTargetName() << "\t\t\t" << vAllActors.at(i).getPortXML() << std::endl;
		}
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		}
		std::cout << "================================================================================" << std::endl;
	}

	bool RTDomainMEM::CheckTargetSlaveManager(std::string sTargetName_tocheck) 
	{
		return !(getSlaveName(sTargetName_tocheck,0).empty());
	}

	//If it does not find a target in the real time domain sets its name to blank.
	int RTDomainMEM::ClearDomain() 
	{
		if(IS_DEBUG) std::cout << "|| RTDomainMEM::ClearDomain ||" << std::endl;
		int iDisabledTargetFound = 0;
		std::string sTN;
		std::cout << "How many actors? " << vAllActors.size() << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			sTN = vAllActors.at(i).getTargetName();
			std::cout << "Target: " << sTN;
			if( sTN.compare("")==0 || !IsThereATarget(sTN) ) {
				std::cout << " DOES NOT exist!" << std::endl;
				iDisabledTargetFound++;
				vAllActors.at(i).setTargetName("");
			}
			else
				std::cout << " exists!" << std::endl;
		}
		if(IS_DEBUG) std::cout << "|| RTDomainMEM:: End of ClearDomain ||" << std::endl;
		return iDisabledTargetFound;
	}

	bool RTDomainMEM::IsTargetPresent(std::string sTN)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if( sTN==vAllActors.at(i).getTargetName())
				return true;
		return false;
	}


} //end namespace rtaixml_domain
