/** \file RTDomainMEM.h 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef RTDOMAINMEM_H_
#define RTDOMAINMEM_H_


/* Class Name:		RTDomainMEM
 * Author: 			Marco Romagnoli
 * Description:	Information about the real time domain. This implementation saves the info on memory (RAM).
 */

#include "ActorDomainRecord.h"
#include "CheckPoint.h"
#include "CheckPointFake.h"
#include "StringTokenizer.h"
#include "Constants.h"
#include "SlaveInfo.h"
#include "RTDomain.h"
#include "XmlRpc.h"
#include "XmlRpcClient.h"

#include <cstdlib>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sys/wait.h>
#include <sys/types.h>

#include <stdio.h>
#include <pthread.h>
#include <sys/poll.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/io.h>
#include <sys/poll.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <vector>
#include <string>

namespace rtaixml_domain
{
  
class RTDomainMEM : public RTDomain
{
	public:
		RTDomainMEM();
		RTDomainMEM(const RTDomainMEM&);
		RTDomainMEM& operator= (const RTDomainMEM&);
		virtual ~RTDomainMEM();
		
	private:
		std::vector<ActorDomainRecord> vAllActors;

		//It provides a free port inside the reserver ones.
		int GetFreePort();
		
	//Services
	public:
		unsigned int NumberOfActiveProcesses();
		//An actor is a pair (slave_server,target) with all the related information
		ActorDomainRecord GetActor(std::string);
		//short: 0 - SlaveName, 1 - SessionId, 2 - TargetName, 3 - PortXML
		ActorDomainRecord GetActor(std::string,short);
		std::string GetFreeSlaveName();
		std::string GetFreeSessionId();
		
		bool AddActor(ActorDomainRecord);
		bool AddActorAndChoosePort(ActorDomainRecord);
		//the parameters are (in order):  sTargetName sScopeName sDigitalName
		bool AddNewActorAndChoosePort(std::string/*,std::string,std::string*/);
		
		bool RemoveActor(ActorDomainRecord);
		//The parameter is the TaskName
		bool RemoveActor(std::string);
		bool RemoveAllActor(ActorDomainRecord);
		
		bool StopRemoveSlaveServer(std::string);
		
		//The parameter is the TaskName
		std::string getSessionId(std::string);
		//The parameter is the TaskName
/*		std::string getTargetIP(std::string);*/
		//The parameter is the Session Id
		std::string getTaskName(std::string);
		//The second paramter is 0 if the first id the TaskName, 1 if the first parameter is the SessionId
		std::string getSlaveName(std::string,short);
		//The second paramter is 0 if the first id the TaskName, 1 if the first parameter is the SessionId
		int getPortXMLSlave(std::string,short);
		//Get a vector containing all the slave names and the xmlrpc port
		std::vector< SlaveInfo > getAllSlaveInfo();
		
		//Print all the available information about the real time domain in a human readable format on the standard output.
		void PrintCurrentStatus();

		//Check if there is already a slave server that manages the selected target. parameter: targetname.
		bool CheckTargetSlaveManager(std::string);
		
		//Check if there is some slaves without an enabled target waiting for client interaction. The target name will be put to blank.
		int ClearDomain();
		
		//Check if the target (taskname) is already present in the real time domain.
		bool IsTargetPresent(std::string);
};

}

#endif /*RTDOMAINMEM_H_*/
