/** \file Digital_Manager.cpp  
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "Digital_Manager.h"

namespace rtaixml_domain {


Digital_Manager::Digital_Manager(int Number_Leds,Target_Leds_T *T_Leds)
{
	Leds=T_Leds;
	Num_Leds = Number_Leds;
	Save_Flag = new int[Num_Leds];
	Trigger_Flag = new int[Num_Leds];
	ResetAll();
	SetAllTrigger();
}

Digital_Manager::~Digital_Manager()
{
	delete[] Save_Flag;
	delete[] Trigger_Flag;
}

int Digital_Manager::Get_Digital(int index)
{
	return Save_Flag[index];
}

void Digital_Manager::Start_Digital(int index)
{
	if(index>=0 && index<Num_Leds)
		Save_Flag[index]=1;
}

void Digital_Manager::Stop_Digital(int index)
{
	if(index>=0 && index<Num_Leds)
		Save_Flag[index]=0;
}

void Digital_Manager::ResetAll()
{
	for (int i = 0; i < Num_Leds; i++)
		Save_Flag[i] = 0;
}

void Digital_Manager::SetAll()
{
	for (int i = 0; i < Num_Leds; i++)
		Save_Flag[i] = 1;
}

unsigned int Digital_Manager::DigitalNumber()
{
	int active_scopes=0;
	for (int i = 0; i < Num_Leds; i++)
		if(Save_Flag[i] == 1)
			active_scopes++;
	return active_scopes;
}

bool Digital_Manager::Get_Trigger(unsigned int index)
{
	if(index>=0 && index<(unsigned int) Num_Leds)
		return Trigger_Flag[index];
	return false;
}

/**
 * Set the trigger mode for the digital specified
 * @param index the number of the digital
 * @param trigger trigger mode on/off
 * @return 
 */
bool Digital_Manager::Set_Trigger(unsigned int index, bool trigger)
{
	if(index>=0 && index<(unsigned int) Num_Leds)
	{
		Trigger_Flag[index]=trigger;
		return true;
	}
	return false;
}

/**
 * Reset all trigger mode to 0 - no trigger.
 */
void Digital_Manager::SetAllTrigger()
{
	for (int i = 0; i < Num_Leds; i++)
		Trigger_Flag[i] = 0;
}

} //end namespace rtaixml_domain

